fn part(p_i: f32) -> f32 {
    p_i * p_i.log2()
}

fn coded_length(table: &[(char, f32, &str, &str)], word: &str) -> u32 {
    let mut length_in_bit = 0;

    for letter in word.chars() {
        let (_l, _p, _f, huffman_word) = table
            .iter()
            .find(|(t_letter, _p, _fano, _huffman)| letter == *t_letter)
            .expect("letter not found in table");

        length_in_bit += huffman_word.len() as u32;
    }

    length_in_bit
}

pub fn emi_main() {
    // word, probability, fano, huffman

    let values = [
        ('V', 0.65, "0", "0"),
        ('Z', 0.11, "100", "100"),
        ('B', 0.05, "101", "1010"),
        ('E', 0.05, "1100", "1011"),
        ('T', 0.04, "1101", "1100"),
        ('>', 0.04, "1110", "1101"),
        ('<', 0.04, "11110", "1110"),
        ('S', 0.02, "11110", "1111"),
    ];

    let mut h = 0.0;
    let mut huffman_l = 0.0;
    let mut fano_l = 0.0;

    for (_name, probability, fano, huffman) in values.iter() {
        h -= part(*probability);
        huffman_l += *probability * huffman.len() as f32;
        fano_l += *probability * fano.len() as f32;
    }

    let huffman_r = huffman_l - h;

    println!("Entropie H = {}", h);
    println!("Mittlere Wortlänge L(Huffman) = {}", huffman_l);
    println!("Mittlere Wortlänge L(fano) = {}", fano_l);
    println!("Redundanz R(Huffman) = {}", huffman_r);

    let code_length = coded_length(&values, "BVVVZVVVVVSSVVE");

    println!("Length in Huffman = {}", code_length);
}
